/*
 * MainBoard_UART.c
*************************************************************************
	Author: Alexander von Lhmann
	
	Hardware
   	processor:		AtMega644
	clock:			20 MHz external RC oscillator

	Function		Pin		Word	Connected With

	UART:
	RXD0			PD0				TX_BT				
	TXD0			PD1				RX_BT
	RTS				PD2				RTS#BT
	CTS				PD3				CTS#BT
	


*************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include "stdbool.h"
#include <string.h>

#define F_CPU 20E6 //Systemtakt in Hz

#include "UART_functions.h"
#include "Mainboard_UART_functions.h"
#include "circbuf.h"
#include <util/delay.h>

// Global VARIABLES
unsigned char byte_buffer;
struct circ_buffer uart_rx_buffer;						// Receive-Circular-Buffer
struct circ_buffer uart_tx_buffer;						// Send-Circular-Buffer


// UART RX-Complete interrupt: 
// Receiving data + buffering (saving)
ISR(USART0_RX_vect) 
{
	unsigned char data;
	data = UDR0;
	buffer_write(&uart_rx_buffer, data);
}

// UART Data register empty interrupt:
// loading new data into UART sending register
ISR(USART0_UDRE_vect) 
{
	if (!buffer_is_empty(&uart_tx_buffer))					// Is buffer filled?
	{
		UDR0 = buffer_read(&uart_tx_buffer);
	}
	else
	{
		UCSR0B &= ~(1<<UDRIE0);							// Disable UDRIE- UART Data Register empty Interrupt 
	}
}

int main(void)
{
	/* Buffer Initialization */
	buffer_init(&uart_rx_buffer);
	buffer_init(&uart_tx_buffer);
	
	/* Variables */
	char controlbyte;
	uint8_t sendbuffer;	
	uint32_t adcrec;
			
	/*********************** Initialization ***********************/
	//Port Directions
	
	DDRA= 0b11011011;				// Custom - please change for your app								
	DDRB= 0b10100110;				// Custom - please change for your app										
	DDRC= 0b01010001;				// Custom - please change for your app										
	DDRD= 0b00001010;				// UART
	
	UART_Init();					// UART initialization
	sei();							// Global Interrupt enable
	
	/*********** MAIN LOOP **************/
    while(1)
    {	
			
		if (!buffer_is_empty(&uart_rx_buffer))						// Received Commands?
		{
			controlbyte = uart_ReceiveByte(&uart_rx_buffer);
			
			switch (controlbyte)
			{
				case 0xAA: {
					sendbuffer =00;
					uart_TransmitByte(&uart_tx_buffer, sendbuffer);
					
					adcrec =  0xFFABCDEF;
					
					sendbuffer =(adcrec >>24);
					uart_TransmitByte(&uart_tx_buffer, sendbuffer);
					sendbuffer =(adcrec >>16);
					uart_TransmitByte(&uart_tx_buffer, sendbuffer);
					sendbuffer =(adcrec >>8);
					uart_TransmitByte(&uart_tx_buffer, sendbuffer);
					sendbuffer =adcrec;
					uart_TransmitByte(&uart_tx_buffer, sendbuffer);		
					sendbuffer =00;
					uart_TransmitByte(&uart_tx_buffer, sendbuffer);													
					break;
				}
				case 0xBB: {
					break;
				}
				case 0xCC:{
					break;					
				}
			}
		}		
    }
}